////-----------------------------------------------------------------------------
////   
////
////   Project: EPA SWMM5
////   Version: 2.0
////   Date:    05/10/22   (Build 2.1)
////   Author:  J. Ferreira (TU Delft)
////
////   Air pocket model development
////-----------------------------------------------------------------------------
//
////-----------------------------------------------------------------------------
//#define _CRT_SECURE_NO_DEPRECATE
//
#include <math.h>
#include "headers.h"
#include "globals.h"
#include "airpocket.h"
#include <float.h>

//
////-----------------------------------------------------------------------------
////  Enumerations
////-----------------------------------------------------------------------------

////-----------------------------------------------------------------------------
////  Data Structures
////-----------------------------------------------------------------------------
//
//
//
////-----------------------------------------------------------------------------
////  Shared Variables
////-----------------------------------------------------------------------------

//////-----------------------------------------------------------------------------
//////  Imported Variables (from SUBCATCH.C)
//////-----------------------------------------------------------------------------


////-----------------------------------------------------------------------------
////  External Functions (prototyped in lid.h)
////-----------------------------------------------------------------------------

//int swmm_runAirInit();
int swmm_runAirTracking();
int swmm_runAirrelease();
int swmm_runAirtransport();
int swmm_runAirmixing();
int swmm_runCreateAirPocket(int AP_inAnalysis[2000], int AP_size);
int swmm_runAirPocketInit();
double airVolumeinPipe(int i);

////-----------------------------------------------------------------------------
//// Local Functions
////-----------------------------------------------------------------------------


//=============================================================================

int swmm_runAirInit()
{

	//modified
	for (int AP = 0; AP < Nobjects[AIRPOCKET]; AP++)

	{
		AirPocket[AP].previousAirVolume = ZERO;
		AirPocket[AP].currentAirVolume = ZERO;
		AirPocket[AP].previousAirPressure = ATM_PRESSURE;
		AirPocket[AP].currentAirPressure = ATM_PRESSURE;
		AirPocket[AP].previousAirDensity = ATM_RHO;
		AirPocket[AP].currentAirDensity = ATM_RHO;
		AirPocket[AP].airReleaseConditions = 0;
		AirPocket[AP].ID = AP;
		AirPocket[AP].previousAP = 0;
		for (int size = 0; size < N_OBJECTS_LINK; size++)
		{
			AirPocket[AP].pipes[size] = -1;
			AirPocket[AP].exits[size] = -1;
		}
	}

	double volume = 0;
	for (int i = 0; i <= Nobjects[NODE]; i++)
	{
		Node[i].connected_to_orif = 0;
	}

	for (int i = 0; i < Nobjects[LINK]; i++)
	{
		if (Link[i].type == CONDUIT)
		{
			volume += airVolumeinPipe(i);
			Link[i].AirPocketnumber = 0;
			Link[i].airpressure = ATM_PRESSURE;
			Link[i].correspondingWaterDepth = 0.0;
			Link[i].airpocketvolume_trapped = airVolumeinPipe(i);
			Link[i].branched = 0;
			Link[i].alreadychecked = 1;
		}
		if (Link[i].type == ORIFICE && Orifice[Link[i].subIndex].hCrit * Orifice[Link[i].subIndex].cDisch > 0.00001)
		{
			Node[Link[i].node1].connected_to_orif = 1;
			Node[Link[i].node2].connected_to_orif = 1;
		}
		else if (Link[i].type == ORIFICE && Orifice[Link[i].subIndex].hCrit * Orifice[Link[i].subIndex].cDisch <= 0.00001)
		{
			Node[Link[i].node1].connected_to_orif = 2;
			Node[Link[i].node2].connected_to_orif = 2;
		}
		else
		{
			if (Node[Link[i].node1].connected_to_orif != 1)
				Node[Link[i].node1].connected_to_orif = 0;
			if (Node[Link[i].node2].connected_to_orif != 1)
				Node[Link[i].node2].connected_to_orif = 0;
		}
	}

	AirPocket[0].previousAirVolume = volume;
	AirPocket[0].currentAirVolume = volume;
	AirPocket[0].previousAirPressure = ATM_PRESSURE;
	AirPocket[0].currentAirPressure = ATM_PRESSURE;
	AirPocket[0].previousAirDensity = ATM_RHO;
	AirPocket[0].currentAirDensity = ATM_RHO;
	AirPocket[0].airReleaseConditions = 0;
	AirPocket[0].ID = 0;
	AirPocket[0].previousAP = 0;
	AirPocketCounter = 0;

	int aux = 0;
	int counter = 0;
	int check = 0;
	int saidascounter = 0;

	for (int i = 0; i < Nobjects[LINK]; i++)
	{
		if ((Node[Link[i].node1].type == OUTFALL || Node[Link[i].node2].type == OUTFALL) && Link[i].type == ORIFICE)
		{
			saidascounter++;
		}
	}

	for (int i = 0; i < Nobjects[LINK]; i++)
	{
		AirPocket[0].pipes[i] = i;
		if (Link[i].type == CONDUIT)
		{
			Link[i].pressurized = 0;
			aux++;
		}

		else if ((Node[Link[i].node1].type == OUTFALL || Node[Link[i].node2].type == OUTFALL) && Link[i].type == ORIFICE)
		{
			AirPocket[0].exits[counter] = i;
			saidascounter--;
			if (Link[i].xsect.yFull > 0.00001)
			{
				AirPocket[0].airReleaseConditions = 0;
				check = 1;
			}
			else if (i == Nobjects[LINK] - 1 && check == 0)
			{
				AirPocket[0].airReleaseConditions = 1;
				check = 0;
			}
		}


	}

	aux = aux - 1;
	AirPocket[0].firstpipe = AirPocket[0].pipes[0];
	AirPocket[0].lastpipe = AirPocket[0].pipes[aux];

	return 0;
}

int swmm_runAirTracking()
{
	int A = 0;
	int initial_node = 0, end_node = 0, Links_checked_counter = 0, ap_size_counter = 0, a = 1;
	int exits[200], exits_counter = 0, to_check = 0;
	int AP_inanalysis[2000], bifurcations = 0, tracking_counter = 0, AP_inanalysis_size = 0;
	int AP_size = 0, n_conduits = 0, pipes[2000], n_pipes = 0;
	int initial = 0;

	///////////////////////////       INITIALIZE              ///////////////////////////
	if (A == 0)
	{
		for (int i = 0; i < 200; i++)
		{
			exits[i] = -1;
		}
		for (int i = 0; i < 2000; i++)
		{
			AP_inanalysis[i] = -1;
		}
		for (int i = 0; i < Nobjects[LINK]; i++)
		{
			if (Link[i].type == CONDUIT)
			{
				Link[i].airvolume = airVolumeinPipe(i);
				if ((Node[Link[i].node1].newDepth >= Link[i].xsect.yFull && Node[Link[i].node2].newDepth >= Link[i].xsect.yFull) || Link[i].newAirPocket == 1)
					Link[i].pressurized = 1;
				else
				{
					n_conduits++;
				}
			}
			to_check++;
			Link[i].checked = 0;
		}
		for (int i = 0; i <= Nobjects[LINK]; i++)
		{
			if (Link[i].type == ORIFICE)
			{
				if (Orifice[Link[i].subIndex].hCrit > 0.000001)
				{
					exits[exits_counter] = i;
					exits_counter++;
				}
			}
		}
	}

	///////////////////////////       Check which air masses are connected together              ///////////////////////////
	while (to_check > 0)
	{
		//get first pipe of entrapped air pocket
		for (int i = 0; i < Nobjects[LINK]; i++)
		{
			//if (Link[i].type==CONDUIT)
			if (Link[i].checked == 0 && Link[i].pressurized == 0 && Link[i].newAirPocket == 0)// && Link[i].type == 0)
			{
				initial = i;
				break;
			}
		}
		int i = initial;

		//check air pockets connections
		for (int j = 0; j <= Nobjects[LINK]; j++)
		{
			// if pipes are connected and the downstream one hasn't been included yet, it tries to do the match
			if ((Link[i].node1 == Link[j].node1 ||
				Link[i].node1 == Link[j].node2 ||
				Link[i].node2 == Link[j].node1 ||
				Link[i].node2 == Link[j].node2) &&
				(i != j) &&
				Link[i].pressurized == 0 && //Link[i].checked == 0 && 
				Link[j].pressurized == 0 && Link[j].checked == 0
				//&& Link[i].type == 0 && Link[j].type == 0
				&& Link[i].newAirPocket == 0 && Link[j].newAirPocket == 0
				)
			{
				//if the upstream pipe hasn't been connected yet it enters this routine, otherwise it has been checked but the code is in a loop to get the branches
				if (Link[i].checked == 0)
				{
					AP_inanalysis[AP_size] = i;
					AP_size++;
					Links_checked_counter++;
					to_check--;
				}
				Link[i].checked = 1;
				i = j;
				j = 0;
			}
			//if link i doesn't find a connection, a new i is considered from the already checked in THIS air pocket
			else if (j == Nobjects[LINK] - 1 && tracking_counter <= AP_size && to_check > 0)
			{
				if (Link[i].checked == 0)
				{
					AP_inanalysis[AP_size] = i;
					AP_size++;
					Links_checked_counter++;
					to_check--;
				}
				Link[i].checked = 1;

				i = AP_inanalysis[tracking_counter];
				tracking_counter++;
				j = 0;
			}
			//is no other link is found to be connected with THIS air pocket
			else if ((j == Nobjects[LINK] - 1 && tracking_counter > AP_size * 1.1) && to_check > 1)
			{
				if (AP_size == 1 || AP_size == 0)
				{
					Link[i].checked = 1;
					to_check--;
					//break;
				}
				else
				{
					//if there are more air pockets, this one takes i as it's last input
					//if (to_check > 1 ainda estou 				AP_inanalysis[AP_size] = i;
					//takes out of the "to check" list all the pipes from this air pocket, even the last input from the previous line
					for (int p = 0; p < Nobjects[LINK]; p++)
					{
						if (AP_inanalysis[p] != -1)
						{
							Link[AP_inanalysis[p]].checked = 1;
						}
					}
					swmm_runCreateAirPocket(AP_inanalysis, AP_size);
					break;
				}
			}

			else if (to_check == 1)
			{
				if (Link[i].checked == 0)
				{
					AP_inanalysis[AP_size] = i;
					AP_size++;
					Link[i].checked = 1;
					Links_checked_counter++;
					to_check--;
				}
			}
		}

		//for now exports the pipes with air/free-surface flow to a file called "pipes_with_air"
		for (int out = 0; out < AP_size; out++)
		{
			fprintf(AirOut2, "%s, ", Link[AP_inanalysis[out]].ID);
			if (out == AP_size - 1)
				fprintf(AirOut2, "\n ");
		}

		//restart air pocket array to eventually check for the next air pocket
		for (int j = 0; j <= AP_size; j++)
		{
			AP_inanalysis[j] = -1;
		}

		//restarts the air pocket size for the next one
		AP_size = 0;
		//check how many more non pressurized pipes and non-checked (not already accounted for in another air pocket) are there in the system
		//if to_check<1, this will be the last process of this routine
		to_check = 0;
		for (int n = 0; n < Nobjects[LINK]; n++)
		{
			if (Link[n].pressurized == 0 && Link[n].checked == 0 && Link[i].newAirPocket == 0)// && Link[n].type == 0)
				to_check++;
		}
	}

	//
	//  Update air pockets data
	//


	for (int i = 0; i <= AirPocketCounter; i++)
	{
		for (int j = 0; j < Nobjects[LINK]; j++)
		{
			if (Link[AirPocket[i].pipes[j]].pressurized == 1)// || Link[AirPocket[i].pipes[j]].newAirPocket == 1)
				AirPocket[i].pipes[j] = -1;
		}
		//AirPocket[i].airReleaseConditions = 2;
	}
	int with_orifice = 0;
	for (int i = 0; i <= AirPocketCounter; i++)
	{
		for (int j = 0; j < N_OBJECTS_LINK; j++)
		{
			if (Link[AirPocket[i].pipes[j]].type == 2)
			{
				double D = Orifice[Link[AirPocket[i].pipes[j]].subIndex].hCrit;
				if (D > 0.000001 && AirPocket[i].airReleaseConditions != 0)
					AirPocket[i].airReleaseConditions = 0;

				if (D <= 0.000001 && AirPocket[i].airReleaseConditions != 0)
					AirPocket[i].airReleaseConditions = 1;
				with_orifice = 1;
			}
			if (j == N_OBJECTS_LINK - 1 && with_orifice == 0)
				AirPocket[i].airReleaseConditions = 2;

		}
	}

	for (int i = 0; i <= AirPocketCounter; i++)
	{
		AirPocket[i].currentAirVolume = 0;
		AirPocket[i].previousAirVolume = 0;
		for (int j = 0; j < N_OBJECTS_LINK; j++)
		{
			if (AirPocket[i].pipes[j] != -1)
			{
				AirPocket[i].previousAirVolume = AirPocket[i].previousAirVolume + airVolumeinPipe(AirPocket[i].pipes[j]);
				AirPocket[i].currentAirVolume = AirPocket[i].currentAirVolume + airVolumeinPipe(AirPocket[i].pipes[j]);
			}
		}
	}

	if (AirPocketCounter > 0)
		AirPocketCounter = AirPocketCounter;

	return 0;
}

int swmm_runAirrelease()
{



	//Initialize parameters
	double Qin = 0.0, Qoutair = 0.0, dQ = 0.0, dpAirdt = 0.0, drhodt = 0.0;
	int saidas[N_OBJECTS_LINK], saidascounter = 0, b = 0;

	//Inicialize the vector for air release pipes
	for (int i = 0; i < sizeof(saidas) / sizeof(saidas[0]); i++)
		saidas[i] = -1;
	//Declare which pipes are air release orifices
	for (int i = 0; i < Nobjects[LINK]; i++)
	{
		if ((Node[Link[i].node1].type == OUTFALL || Node[Link[i].node2].type == OUTFALL) && Link[i].type == ORIFICE)
		{
			saidas[saidascounter] = i;
			saidascounter++;
		}
	}

	saidascounter = Nnodes[OUTFALL];

	//For all air pockets:
	for (int i = 0; i <= AirPocketCounter; i++)
	{
		int testing[N_OBJECTS_LINK], testing_size = 0;
		for (int aux1 = 0; aux1 < N_OBJECTS_LINK; aux1++)
		{
			if (AirPocket[i].pipes[aux1] != -1)
			{
				AirPocket[i].firstpipe = AirPocket[i].pipes[aux1];
				break;
			}
		}
		if (AirPocket[i].airReleaseConditions == 1) // for dead ends
			Qin = Link[0].newFlow;
		else
			for (int k = 0; k < Nobjects[LINK]; k++)
			{
				if (AirPocket[i].pipes[k] != -1 && Link[AirPocket[i].pipes[k]].type == 0)
					// for the others since flow would go backwards if only Link[].newFlow and the air would depressurize without reason

					if (((Node[Link[AirPocket[i].pipes[k]].node1].newDepth > Link[AirPocket[i].pipes[k]].xsect.yFull && Node[Link[AirPocket[i].pipes[k]].node2].newDepth < Link[AirPocket[i].pipes[k]].xsect.yFull)
						|| (Node[Link[AirPocket[i].pipes[k]].node1].newDepth < Link[AirPocket[i].pipes[k]].xsect.yFull && Node[Link[AirPocket[i].pipes[k]].node2].newDepth > Link[AirPocket[i].pipes[k]].xsect.yFull)) && Link[AirPocket[i].pipes[k]].newAirPocket == 0)
					{
						for (int kk = 0; kk < Nobjects[LINK]; kk++)
						{
							if ((Link[k].node1 == Link[kk].node1 ||
								Link[k].node1 == Link[kk].node2 ||
								Link[k].node2 == Link[kk].node1 ||
								Link[k].node2 == Link[kk].node2) &&
								(k != kk) &&
								(Link[kk].pressurized == 1 || Link[kk].newAirPocket == 1))
							{
								testing[testing_size] = kk;
								testing_size++;
								for (int check = 0; check < testing_size; check++)
								{
									if (testing[check] == kk && check != 0)
										testing[check] = -1;
								}
								break;
							}
						}
						//Qin = Qin + (Node[Link[AirPocket[i].pipes[k]].node2].outflow + Node[Link[AirPocket[i].pipes[k]].node1].outflow) / 2;
						//Qin += Link[AirPocket[i].pipes[k]].newFlow;
					}
			}
		for (int kk = 0; kk < testing_size; kk++)
		{
			double Qtoadd = 0;

			//Qtoadd = Link[testing[kk]].newFlow;
			Qtoadd = MAX(Node[Link[testing[kk]].node1].inflow, Node[Link[testing[kk]].node1].outflow);
			Qtoadd = MAX(Qtoadd, Node[Link[testing[kk]].node2].inflow);
			Qtoadd = MAX(Qtoadd, Node[Link[testing[kk]].node2].outflow);
			Qin += Qtoadd / testing_size;
		}


		//for the air pockets with air release from the system :
		if (AirPocket[i].airReleaseConditions == 0)
		{

			double D = 0.0;
			//for all the possible air release devices from the system, calculate their air release flow rate
			for (int j = 0; j < saidascounter; j++)
			{
				// if orifice is circular it is taken as a circular air release orifice in the pipe
				if (Link[saidas[j]].xsect.type == CIRCULAR && Orifice[Link[saidas[j]].subIndex].hCrit > 0.0001)
				{
					double Qair = 0.0;
					D = Orifice[Link[saidas[j]].subIndex].hCrit;
					double area = pow(D, 2.0) / 4.0 * PI;
					double Y1, Y2, Y3, Y4, Y;

					//chocked conditions
					if (AirPocket[i].previousAirPressure / ATM_PRESSURE >= 1.89)
					{
						Qair = Orifice[Link[saidas[j]].subIndex].cDisch * pow(D, 2.0) / 4.0 * PI * sqrt(GRAVITY * 1000 / AirPocket[i].previousAirDensity * (AirPocket[i].currentAirPressure / 9810) * 3.28084) * sqrt(N_AIR * pow((2.0 / (N_AIR + 1.0)), ((N_AIR + 1.0) / (N_AIR - 1.0))));
						Y1 = 1.0;
						Y2 = 1.0;
						Y3 = 1.0;
						Y4 = 1.0;
						Y = 1.0;
					}

					//non-chocked conditions
					else if (AirPocket[i].previousAirPressure / ATM_PRESSURE < 1.89 && AirPocket[i].previousAirPressure >= ATM_PRESSURE)
					{
						double aux = ATM_PRESSURE / AirPocket[i].previousAirPressure;
						Y1 = N_AIR / (N_AIR - 1.0);
						Y2 = pow(aux, (2.0 / N_AIR));
						Y3 = (1.0 - pow(aux, (N_AIR - 1.0) / N_AIR));
						Y4 = (1.0 - aux);

						if (Y4 == 0.0)
							Y = 0.0;
						else
							Y = sqrt(Y1 * Y2 * Y3 / Y4);
						Qair = Orifice[Link[saidas[j]].subIndex].cDisch * pow(D, 2) / 4.0 * PI * Y * sqrt(2.0 * GRAVITY * 1000 / AirPocket[i].currentAirDensity * (((AirPocket[i].currentAirPressure + AirPocket[i].previousAirPressure) / 2.0 - ATM_PRESSURE) * 3.28084 / 9810));
						if (Qair < 0.0)
							Qair = 0.0;
					}
					//if negative pressures, the system does not allow air intake
					else if (AirPocket[i].currentAirPressure < ATM_PRESSURE)
						Qair = 0.0;
					Qoutair += Qair;
				}
				//else it is taken as an air valve with the same diameter
				else if (Link[saidas[j]].xsect.type == RECT_CLOSED && Orifice[Link[saidas[j]].subIndex].hCrit > 0.0001)
				{
					double Qair = 0.0;
					D = Orifice[Link[saidas[j]].subIndex].hCrit;
					double area = pow(D, 2.0) / 4.0 * PI;
					if (AirPocket[i].currentAirPressure < ATM_PRESSURE / 0.53 && AirPocket[i].currentAirPressure > ATM_PRESSURE)
					{
						Qair = Orifice[Link[saidas[j]].subIndex].cDisch * area * ((AirPocket[i].currentAirPressure + AirPocket[i].previousAirPressure) / 2.0 * 3.28084 / 9810) * sqrt(7 / (AIR_R * AIR_TEMP) * (pow(ATM_PRESSURE / (AirPocket[i].currentAirPressure + AirPocket[i].previousAirPressure) / 2.0, 1.4286) - pow(ATM_PRESSURE / (AirPocket[i].currentAirPressure + AirPocket[i].previousAirPressure) / 2.0, 1.714)));
					}
					else if (AirPocket[i].currentAirPressure > ATM_PRESSURE / 0.53)
					{
						Qair = Orifice[Link[saidas[j]].subIndex].cDisch * area * 0.686 * ((AirPocket[i].currentAirPressure + AirPocket[i].previousAirPressure) / 2.0 * 3.28084 / 9810) / (AIR_R * AIR_TEMP);
					}
					else if (AirPocket[i].currentAirPressure <= ATM_PRESSURE)
						Qair = 0.0;
					Qoutair += Qair;
				}
			}
			if (Qoutair < 0.0)
				Qoutair = 0.0;
		}
		else
			Qoutair = 0.0;


		//for all the air pockets that are not considered as entrapped, i.e., air pockets with air release or in dead ends
		//calculate air pressure and density. If any of these become lower than at atmospheric pressure they are the atmospheric ones. similar boundary as air release flow rate
		if (AirPocket[i].currentAirVolume > MIN_AIR && AirPocket[i].airReleaseConditions != 2)
		{
			//balance between water flow rate going into the pipe and air flow rate being released
			dQ = (Qin - Qoutair);

			printf("\nQin = %f\n Air Pocket Counter= %d \n", Qin, AirPocketCounter);
			for (int out = 0; out <= AirPocketCounter; out++)
			{
				fprintf(AirOut, "%f, %f,", Qin, Qoutair);
			}

			//air pressure and density calculations
			dpAirdt = AirPocket[i].previousAirPressure * N_AIR / AirPocket[i].currentAirVolume * dQ;
			drhodt = AirPocket[i].previousAirDensity / AirPocket[i].currentAirVolume * dQ;
			AirPocket[i].currentAirPressure = AirPocket[i].previousAirPressure + dpAirdt * (NewRoutingTime - OldRoutingTime) / 1000;
			AirPocket[i].currentAirDensity = AirPocket[i].previousAirDensity + drhodt * (NewRoutingTime - OldRoutingTime) / 1000;

			//minimum threshold
			if (AirPocket[i].currentAirPressure < ATM_PRESSURE)
				AirPocket[i].currentAirPressure = ATM_PRESSURE;
			if (AirPocket[i].currentAirDensity < ATM_RHO)
				AirPocket[i].currentAirDensity = ATM_RHO;
			if (AirPocket[i].currentAirVolume < 0.0)
				AirPocket[i].currentAirVolume = ZERO;

			//update previous time step
			AirPocket[i].previousAirPressure = AirPocket[i].currentAirPressure;
			AirPocket[i].previousAirVolume = AirPocket[i].currentAirVolume;
			AirPocket[i].previousAirDensity = AirPocket[i].currentAirDensity;

		}
		//if the air pocket has already been mostly release and a very small volume remains
		else if (AirPocket[i].airReleaseConditions != 2)
		{
			AirPocket[i].previousAirPressure = AirPocket[i].currentAirPressure;
			AirPocket[i].currentAirPressure = AirPocket[i].currentAirPressure;
			AirPocket[i].currentAirVolume = ZERO;
			AirPocket[i].previousAirVolume = 0.0;
			AirPocket[i].currentAirDensity = AirPocket[i].currentAirDensity;
			AirPocket[i].previousAirDensity = AirPocket[i].currentAirDensity;
		}
		else if (AirPocket[i].airReleaseConditions == 2)
		{
			AirPocket[i].previousAirPressure = AirPocket[i].currentAirPressure;
			double nodetoconsidermax = Node[Link[AirPocket[i].firstpipe].node1].newDepth + Node[Link[AirPocket[i].firstpipe].node1].invertElev,
				nodetoconsidermin = Node[Link[AirPocket[i].firstpipe].node1].newDepth + Node[Link[AirPocket[i].firstpipe].node1].invertElev;

			nodetoconsidermax = MAX(nodetoconsidermax, Node[Link[AirPocket[i].firstpipe].node2].newDepth + Node[Link[AirPocket[i].firstpipe].node2].invertElev);
			nodetoconsidermax = MAX(nodetoconsidermax, Node[Link[AirPocket[i].lastpipe].node1].newDepth + Node[Link[AirPocket[i].lastpipe].node1].invertElev);
			nodetoconsidermax = MAX(nodetoconsidermax, Node[Link[AirPocket[i].lastpipe].node2].newDepth + Node[Link[AirPocket[i].lastpipe].node2].invertElev);

			nodetoconsidermin = MAX(nodetoconsidermin, Node[Link[AirPocket[i].firstpipe].node2].newDepth + Node[Link[AirPocket[i].firstpipe].node2].invertElev);
			nodetoconsidermin = MAX(nodetoconsidermin, Node[Link[AirPocket[i].lastpipe].node1].newDepth + Node[Link[AirPocket[i].lastpipe].node1].invertElev);
			nodetoconsidermin = MAX(nodetoconsidermin, Node[Link[AirPocket[i].lastpipe].node2].newDepth + Node[Link[AirPocket[i].lastpipe].node2].invertElev);

			double elevation = (Node[Link[AirPocket[i].x_mass_pipe].node2].invertElev + Node[Link[AirPocket[i].x_mass_pipe].node1].invertElev) / 2;

			AirPocket[i].currentAirPressure = ((ATM_PRESSURE + ((nodetoconsidermax + nodetoconsidermin) / 2.0 - elevation) * MperFT * 9800.0)); //(nodetoconsidermax- nodetoconsidermin)/2 * MperFT * 9800;
			AirPocket[i].previousAirVolume = AirPocket[i].currentAirVolume;
			AirPocket[i].currentAirVolume = pow(pow(AirPocket[i].currentAirVolume, N_AIR) * AirPocket[i].previousAirPressure / AirPocket[i].currentAirPressure, 1 / N_AIR);
			int aux = 0;
			double APlength = 0;
			for (int counter = 0; counter <= Nobjects[LINK]; counter++)
			{
				if (AirPocket[i].pipes[counter] != -1)
				{
					APlength += Conduit[Link[AirPocket[i].pipes[counter]].subIndex].modLength;
					aux++;
				}
				else
					break;
			}

			for (int size = 0; size <= AirPocket[i].npipes; size++)
			{
				if (AirPocket[i].pipes[size] == -1)
					break;
				Link[AirPocket[i].pipes[size]].correspondingWaterDepth = xsect_getYofA(&Link[AirPocket[i].pipes[size]].xsect, ((Link[AirPocket[i].pipes[size]].xsect.aFull * Conduit[AirPocket[i].pipes[size]].length) - AirPocket[i].currentAirVolume / AirPocket[i].npipes) / Conduit[AirPocket[i].pipes[size]].length); //area de seco com ar
			}

			//minimum threshold
			if (AirPocket[i].currentAirPressure < ATM_PRESSURE)
				AirPocket[i].currentAirPressure = ATM_PRESSURE;
			if (AirPocket[i].currentAirDensity < ATM_RHO)
				AirPocket[i].currentAirDensity = ATM_RHO;
			if (AirPocket[i].currentAirVolume < 0.0)
				AirPocket[i].currentAirVolume = ZERO;
		}
	}
	return 0;
}

int swmm_runAirtransport()
{
	int aux = 0;
	double APlength = 0;
	int  n_pipes_bif = 0;
	double total_air = 0.0;

	for (int i = AirPocketCounter; i >= 0; i--)
	{
		if (AirPocket[i].airReleaseConditions == 2)
		{
			for (int counter = 0; counter <= AirPocket[i].npipes; counter++)
			{
				if (AirPocket[i].pipes[counter] != -1)
				{
					APlength += Conduit[Link[AirPocket[i].pipes[counter]].subIndex].modLength;
					aux++;
				}
				else
					break;
			}

			//Air pocket transport
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			double z1 = Node[Link[AirPocket[i].firstpipe].node2].invertElev;
			double z2 = Node[Link[AirPocket[i].lastpipe].node1].invertElev;
			double S = (z1 - z2) / APlength;

			double theta = atan(S);
			double Vc = 0; // (0.55 * sqrt(sin(S)) + 0.53)* sqrt(SI_GRAVITY * Link[AirPocket[i].x_mass_pipe].xsect.yFull); // Critical flow velocity from Escarameia (2004) HR Wallingford data
			double Vap = 0.0, V = 0.0;
			double limit1 = 1.25;
			double limit2 = 4.25;

			int model = 0; // Escarameia = 0; Drag = 1

			//Escarameia 2004
			if (model == 0)
			{
				if (S < limit1)
					Vap = 1.655 * Vc - 0.556;
				else if (S >= limit1 && S < limit2)
					Vap = 0.244 * Vc - 0.551;
				else if (S >= limit2)
					Vap = 0.319 * Vc - 0.223;

				//if (S == 0.0)
				Vap = Link[AirPocket[i].firstpipe].newFlow / Link[AirPocket[i].firstpipe].xsect.aFull;
				V = Vap / 2 / MperFT;
			}
			//Drag coefficients
			if (model == 1)
			{
				double Drag = 0.4;
				Vap = sqrt(round(4.0 / 3.0) * GRAVITY * Link[AirPocket[i].x_mass_pipe].xsect.yFull * sin(theta) / Drag);
				//if going down
				if (Node[Link[AirPocket[i].firstpipe].node2].invertElev - Node[Link[AirPocket[i].lastpipe].node1].invertElev >= 0.0)
					V = Link[AirPocket[i].x_mass_pipe].newFlow / Link[AirPocket[i].x_mass_pipe].xsect.aFull - Vap;
				//if going up
				else
					V = Link[AirPocket[i].x_mass_pipe].newFlow / Link[AirPocket[i].x_mass_pipe].xsect.aFull + Vap;
				V = V / 10.0;
			}

			//Critical flow velocity and advancement of the air pocket
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			if ((Link[AirPocket[i].x_mass_pipe].newFlow / Link[AirPocket[i].x_mass_pipe].xsect.aFull >= Vc && model == 0) || model == 1)
			{
				if (Node[Link[AirPocket[i].firstpipe].node2].invertElev - Node[Link[AirPocket[i].lastpipe].node1].invertElev <= 0.0)
				{
					//advance of the air pocket centre of mass
					AirPocket[i].x_mass = AirPocket[i].x_mass + V * (NewRoutingTime - OldRoutingTime) / 1000;

					//if the air pocket centre of mass moves to another pipe, this is reassinged in the AirPocket object
					if (AirPocket[i].x_mass > Conduit[AirPocket[i].x_mass_pipe].length || AirPocket[i].x_mass < 0)
					{
						//from the utmost downstream AP to the utmost upstream	

							//if in the direction of the pipe orientation
						if (AirPocket[i].x_mass > Conduit[AirPocket[i].x_mass_pipe].length)
						{
							for (int m = AirPocket[i].npipes; m >= 0; m--)
							{
								for (int k = 0; k <= Nobjects[LINK]; k++)
								{
									//move the pipe to the next pipe in the direction of the flow
									if ((((Link[AirPocket[i].pipes[m]].node2 == Link[k].node1 ||
										Link[AirPocket[i].pipes[m]].node2 == Link[k].node2) &&
										Link[AirPocket[i].pipes[m]].newFlow > 0)
										||
										((Link[AirPocket[i].pipes[m]].node1 == Link[k].node1 ||
											Link[AirPocket[i].pipes[m]].node1 == Link[k].node2) &&
											Link[AirPocket[i].pipes[m]].newFlow < 0))
										&&
										AirPocket[i].pipes[m] != k)
									{
										Link[k].correspondingWaterDepth = Link[AirPocket[i].pipes[m]].correspondingWaterDepth; //change to be according to the pipe diameter and not the simple movement
										Link[AirPocket[i].pipes[m]].correspondingWaterDepth = 0.0;
										Link[AirPocket[i].pipes[m]].AirPocketnumber = 0;

										AirPocket[i].pipes[m] = k;
										Link[AirPocket[i].pipes[m]].AirPocketnumber = i;
										break;
									}
								}

								AirPocket[i].firstpipe = AirPocket[i].pipes[0];
								AirPocket[i].lastpipe = AirPocket[i].pipes[AirPocket[i].npipes];

								if (AirPocket[i].npipes % 2 == 0)
									AirPocket[i].x_mass_pipe = AirPocket[i].pipes[AirPocket[i].npipes / 2 + 1];
								else
									AirPocket[i].x_mass_pipe = AirPocket[i].pipes[AirPocket[i].npipes / 2];

							}

							AirPocket[i].firstpipe = AirPocket[i].pipes[0];
							AirPocket[i].lastpipe = AirPocket[i].pipes[AirPocket[i].npipes];

							if (AirPocket[i].npipes % 2 == 0)
								AirPocket[i].x_mass_pipe = AirPocket[i].pipes[AirPocket[i].npipes / 2 + 1];
							else
								AirPocket[i].x_mass_pipe = AirPocket[i].pipes[AirPocket[i].npipes / 2];


						}
						//otherwise, 
						if (AirPocket[i].x_mass < 0)
						{
							for (int m = 0; m < AirPocket[i].npipes; m++)
							{
								for (int k = 0; k <= Nobjects[LINK]; k++)
								{
									//move the pipe to the previous pipe in the direction of the flow
									if ((Link[AirPocket[i].pipes[m]].node2 == Link[k].node1 ||
										Link[AirPocket[i].pipes[m]].node2 == Link[k].node2) &&
										AirPocket[i].pipes[m] != k)
									{
										Link[k].correspondingWaterDepth = Link[AirPocket[i].pipes[m]].correspondingWaterDepth;
										if (m == 0)
											Link[AirPocket[i].pipes[m]].correspondingWaterDepth = 0.0;
										AirPocket[i].pipes[m] = k;
										break;
									}
								}

								if (m == 1)
									AirPocket[i].firstpipe = AirPocket[i].pipes[1];
								AirPocket[i].firstpipe = AirPocket[i].pipes[AirPocket[i].npipes];
								if (AirPocket[i].npipes % 2 == 0)
									AirPocket[i].x_mass_pipe = AirPocket[i].pipes[AirPocket[i].npipes / 2 + 1];
								else
									AirPocket[i].x_mass_pipe = AirPocket[i].pipes[AirPocket[i].npipes / 2];
							}
						}

						//reset centre of mass position for this air pocket
						if (AirPocket[i].x_mass < 0)
							AirPocket[i].x_mass = Conduit[AirPocket[i].firstpipe].length;
						else if (AirPocket[i].x_mass > Conduit[AirPocket[i].x_mass_pipe].length)
							AirPocket[i].x_mass = 0;
					}
				}
			}
		}
	}
	return 0;
}

int swmm_runAirmixing()
{
	//Initialize parameters
	int aux = 0,
		next = 0;
	double volume = 0.0,
		APlength = 0.0,
		Qair_Qwat = 0.0,
		mixed_volume = 0.0,
		flow_velocity = 0.0,
		depth = 0.0,
		total = 0.0;

	//from all the air pockets
	for (int i = 0; i <= AirPocketCounter; i++)
	{
		double froude = fabs(Link[AirPocket[i].firstpipe].newFlow / Link[AirPocket[i].firstpipe].xsect.aFull) / sqrt(Link[AirPocket[i].firstpipe].xsect.yFull * GRAVITY);
		if (AirPocket[i].airReleaseConditions == 2 && AirPocket[i].currentAirVolume > 0.0 && Node[Link[AirPocket[i].firstpipe].node2].invertElev - Node[Link[AirPocket[i].lastpipe].node1].invertElev >= 0.0 && Node[Link[AirPocket[i].lastpipe].node1].invertElev > 0.0000001)
		{
			fprintf(Froude, ",%10.20lf ", froude);
		}
		//for those entrapped 
		double caso = Node[Link[AirPocket[i].firstpipe].node2].invertElev - Node[Link[AirPocket[i].lastpipe].node1].invertElev;
		if (AirPocket[i].airReleaseConditions == 2 && AirPocket[i].currentAirVolume > 0.0 && Node[Link[AirPocket[i].firstpipe].node2].invertElev - Node[Link[AirPocket[i].lastpipe].node1].invertElev >= 0.0 && froude > 1.01)
		{
			//if the air pocket is in a donwwards sloped or horizontal pipe process some mixing
				//correlation between Qair and Qwater Escarameia (2004)

			double LOW_a = Node[0].pondedArea * pow(MperFT, 2);
			double HIGH_a = Node[1].pondedArea * pow(MperFT, 2);

			double LOW_b = Node[2].pondedArea * pow(MperFT, 2);
			double HIGH_b = Node[3].pondedArea * pow(MperFT, 2);

			double extra = 0;
			double factor = LOW_a,
				exponent = LOW_b;

			extra = pow(froude - 1.0, exponent);
			double theta = 0.0;
			Qair_Qwat = factor * 0.0025 * extra + theta;
			Qair_Qwat = Qair_Qwat;

			fprintf(Froude, ",%10.20lf, ", Qair_Qwat);
			//mixing volume at each dt
			double added_volume = 0.0;

			double initial_percentage = 1;

			if (Link[AirPocket[i].firstpipe].airpocketvolume_trained != 0.0)
			{
				added_volume = Link[AirPocket[i].firstpipe].airpocketvolume_trained * initial_percentage;
				Link[AirPocket[i].firstpipe].airpocketvolume_trained = 0.0;
			}
			double water = (Node[Link[AirPocket[i].firstpipe].node2].inflow + Node[Link[AirPocket[i].firstpipe].node1].inflow) / 2 * (NewRoutingTime - OldRoutingTime) / 1000;
			mixed_volume = Qair_Qwat * water;
			//mixed_volume = 0.0;

			//discount it from air pocket volume
			double total_AV = 0.0;
			for (int l = 0; l < AirPocket[i].npipes; l++)
			{
				total_AV = total_AV + (Conduit[l].length * pow(Link[i].xsect.yFull, 2) * PI / 4);
			}
			if (AirPocket[i].currentAirVolume - mixed_volume + added_volume < 0.9 * total_AV)
			{
				AirPocket[i].currentAirVolume = AirPocket[i].currentAirVolume - mixed_volume + added_volume;
			}
			else
			{
				AirPocket[i].currentAirVolume = AirPocket[i].currentAirVolume - mixed_volume;
			}
			if (AirPocket[i].currentAirVolume <= 0.0)
				AirPocket[i].currentAirVolume = ZERO;


			int aux = 0;
			double APlength = 0;
			for (int counter = 0; counter <= Nobjects[LINK]; counter++)
			{
				if (AirPocket[i].pipes[counter] != -1)
				{
					APlength += Conduit[Link[AirPocket[i].pipes[counter]].subIndex].modLength;
					aux++;
				}
				else
					break;
			}

			for (int size = 0; size <= AirPocket[i].npipes; size++)
			{
				if (AirPocket[i].pipes[size] != -1)
				{
					if (((Link[AirPocket[i].pipes[size]].xsect.aFull * Conduit[AirPocket[i].pipes[size]].length) - AirPocket[i].currentAirVolume / AirPocket[i].npipes) / Conduit[AirPocket[i].pipes[size]].length < 0.0)
						Link[i].correspondingWaterDepth = Link[i].xsect.yFull;
					else
						Link[AirPocket[i].pipes[size]].correspondingWaterDepth = xsect_getYofA(&Link[AirPocket[i].pipes[size]].xsect, ((Link[AirPocket[i].pipes[size]].xsect.aFull * Conduit[AirPocket[i].pipes[size]].length) - AirPocket[i].currentAirVolume / AirPocket[i].npipes) / Conduit[AirPocket[i].pipes[size]].length); //area de seco com ar
				}
				else
					break;
			}
		}
		AirPocket[i].previousAirVolume = AirPocket[i].currentAirVolume;
	}
	//movement of the bubbly flow
	for (int m = 0; m < Nobjects[LINK]; m++)
	{
		if (Link[m].airpocketvolume_trained > 0.0 && fabs(Link[m].newFlow) >= 0)
		{
			//assuming bubbly flow moves at the same velocity as the flow
			flow_velocity = fabs(Link[m].newFlow / Link[m].xsect.aFull);
			//only if there is some bubbly flow on that pipe

			//bubbly flow centre of mass moves dx with flow velocity
			double air_velocity = Node[3].pondedArea * pow(MperFT, 2);
			if (air_velocity < 0.1)
				air_velocity = 0.2;
			Link[m].trained_x_centre = Link[m].trained_x_centre + flow_velocity * (NewRoutingTime - OldRoutingTime) / 1000 * air_velocity;

			//if the air pocket centre of mass moves to another pipe, this is reassinged in the AirPocket object
			if (Link[m].trained_x_centre > Conduit[m].length || Link[m].trained_x_centre < 0)
			{
				//from the utmost downstream AP to the utmost upstream	

				//if in the direction of the pipe orientation
				if (Link[m].trained_x_centre > Conduit[m].length)
				{

					for (int k = 0; k <= Nobjects[LINK]; k++)
					{
						//move the pipe to the next pipe in the direction of the flow
						if ((((Link[m].node2 == Link[k].node1 ||
							Link[m].node2 == Link[k].node2) &&
							Link[m].newFlow > 0)
							||
							((Link[m].node1 == Link[k].node1 ||
								Link[m].node1 == Link[k].node2) &&
								Link[m].newFlow < 0))
							&&
							m != k)
						{
							Link[k].airpocketvolume_trained = Link[m].airpocketvolume_trained; //change to be according to the pipe diameter and not the simple movement
							Link[m].airpocketvolume_trained = 0.0;
							next = k;
							break;
						}
						if (k == Nobjects[LINK])
							break;
					}

				}
				//otherwise, 
				if (Link[m].trained_x_centre < 0)
				{
					for (int k = 0; k <= Nobjects[LINK]; k++)
					{
						//move the pipe to the previous pipe in the direction of the flow
						if ((((Link[m].node2 == Link[k].node1 ||
							Link[m].node2 == Link[k].node2) &&
							Link[m].newFlow < 0)
							||
							((Link[m].node1 == Link[k].node1 ||
								Link[m].node1 == Link[k].node2) &&
								Link[m].newFlow > 0))
							&&
							m != k)
						{
							Link[k].airpocketvolume_trained = Link[m].airpocketvolume_trained; //change to be according to the pipe diameter and not the simple movement
							Link[m].airpocketvolume_trained = 0.0;
							next = k;
							break;
						}
						if (k == Nobjects[LINK])
							break;
					}
				}
				//reset centre of mass position for this air pocket
				if (Link[m].trained_x_centre < 0)
				{
					total = Link[m].trained_x_centre;
					Link[m].trained_x_centre = 0.0;
					Link[next].trained_x_centre = Conduit[m].length - total;
					m = next;
				}
				else if (Link[m].trained_x_centre > Conduit[m].length)
				{
					total = Link[m].trained_x_centre;
					Link[m].trained_x_centre = 0.0;
					Link[next].trained_x_centre = total - Conduit[m].length;
					m = next;
				}
			}
		}
	}
	return 0;
}

int swmm_runCreateAirPocket(int AP_inAnalysis[2000], int AP_size)
{
	int npipes = 0;
	AirPocketCounter++;
	swmm_runAirPocketInit();
	for (int i = 0; i < AP_size; i++)
	{
		printf("%d, ", AP_inAnalysis[i]);
	}
	//
	//	Creates new air pocket in the system and erases duplicate pipes from other air pockets, i.e., when an air pocket is split, 
	//	the pipes spilt from the previous air pocket neew to be removed
	//

	for (int size = 0; size < AP_size; size++)
	{
		int previous = 0;

		// 
		// erase duplicate pipes from other air pockets, i.e., when an air pocket is split, the pipes spilt from the previous air pocket neew to be removed
		//

		for (int i = 0; i <= AirPocketCounter; i++)
		{
			if (i != AirPocketCounter)
			{
				for (int lenght = 0; lenght < Nobjects[LINK]; lenght++)
				{
					if (AirPocket[i].pipes[lenght] == AP_inAnalysis[size])
					{
						previous = i;
						AirPocket[i].pipes[lenght] = -1;
						break;
					}
				}
			}
			else
			{
				for (int lenght = 0; lenght < Nobjects[LINK]; lenght++)
				{
					AirPocket[i].pipes[size] = AP_inAnalysis[size];
					AirPocket[i].previousAP = previous;
					break;
				}
			}
		}
	}

	//
	//	Determine air pockets number of pipes and update them
	//

	for (int i = 0; i <= AirPocketCounter; i++)
	{
		for (int j = 0; j < N_OBJECTS_LINK; j++)
		{
			if (AirPocket[i].pipes[j] != -1)
			{
				npipes++;
			}
		}
		AirPocket[i].npipes = npipes;
		npipes = 0;
	}

	//
	// Reassign air pocket volume for each air pocket based on the pipes where it is contained
	//

	for (int i = 0; i <= AirPocketCounter; i++)
	{
		AirPocket[i].currentAirVolume = 0;
		AirPocket[i].previousAirVolume = 0;
		for (int j = 0; j < N_OBJECTS_LINK; j++)
		{
			if (AirPocket[i].pipes[j] != -1)
			{
				AirPocket[i].previousAirVolume = AirPocket[i].previousAirVolume + airVolumeinPipe(AirPocket[i].pipes[j]);
				AirPocket[i].currentAirVolume = AirPocket[i].currentAirVolume + airVolumeinPipe(AirPocket[i].pipes[j]);
			}
		}
	}

	// 
	// Assign new air release conditions to all the air pockets after pipes have been discarded
	//
	int with_orifice = 0;
	for (int i = 0; i <= AirPocketCounter; i++)
	{
		for (int j = 0; j < AirPocket[i].npipes; j++)
		{
			if (Link[AirPocket[i].pipes[j]].type == ORIFICE)
			{
				double D = Orifice[Link[AirPocket[i].pipes[j]].subIndex].hCrit;
				if (D > 0.000001 && AirPocket[i].airReleaseConditions != 0)
					AirPocket[i].airReleaseConditions = 0;

				if (D <= 0.000001 && AirPocket[i].airReleaseConditions != 0)
					AirPocket[i].airReleaseConditions = 1;
				with_orifice = 1;
			}
			if (j == AirPocket[i].npipes - 1 && with_orifice == 0)
				AirPocket[i].airReleaseConditions = 2;
		}
	}

	//
	// Assign entrapped air pocket centre of mass
	//

	if (AirPocket[AirPocketCounter].airReleaseConditions == 2)
	{
		AirPocket[AirPocketCounter].firstpipe = AirPocket[AirPocketCounter].pipes[0];
		AirPocket[AirPocketCounter].lastpipe = AirPocket[AirPocketCounter].pipes[AirPocket[AirPocketCounter].npipes - 1];
		if (AirPocket[AirPocketCounter].npipes % 2 == 0)
			AirPocket[AirPocketCounter].x_mass_pipe = AirPocket[AirPocketCounter].pipes[AirPocket[AirPocketCounter].npipes / 2 + 1];
		else
			AirPocket[AirPocketCounter].x_mass_pipe = AirPocket[AirPocketCounter].pipes[AirPocket[AirPocketCounter].npipes / 2];
		Link[AirPocket[AirPocketCounter].lastpipe].airpocketvolume_trained = 0.0;
		Link[AirPocket[AirPocketCounter].lastpipe].trained_x_centre = 0.0;
	}

	//
	// Pass on density and pressure data
	//

	if (AirPocket[AirPocketCounter].airReleaseConditions != 2)
	{
		AirPocket[AirPocketCounter].previousAirDensity = AirPocket[AirPocket[AirPocketCounter].previousAP].currentAirDensity;
		AirPocket[AirPocketCounter].currentAirDensity = AirPocket[AirPocket[AirPocketCounter].previousAP].currentAirDensity;
		AirPocket[AirPocketCounter].previousAirPressure = AirPocket[AirPocket[AirPocketCounter].previousAP].currentAirPressure;
		AirPocket[AirPocketCounter].currentAirPressure = AirPocket[AirPocket[AirPocketCounter].previousAP].currentAirPressure;
	}

	if (AirPocket[AirPocketCounter].airReleaseConditions == 0 && AirPocket[AirPocketCounter].npipes < 6)
	{
		for (int i = 0; i < AirPocket[AirPocketCounter].npipes; i++)
		{
			Link[AirPocket[AirPocketCounter].pipes[i]].pressurized = 1;
		}
	}

	//
	// Pass on attributes from air pockets to each pipe from the air pockets and 
	//

	for (int a = 0; a <= AP_size; a++)
	{
		Link[AP_inAnalysis[a]].AirPocketnumber = AirPocketCounter;
		Link[AP_inAnalysis[a]].airpocketvolume_trapped = 0;
		Link[AP_inAnalysis[a]].correspondingWaterDepth = 0;
		Link[AP_inAnalysis[a]].newAirPocket = 0;
	}

	//
	//  
	//

	int i = AirPocketCounter;

	if (AirPocket[AirPocketCounter].airReleaseConditions == 2)
	{

		double nodetoconsidermax = Node[Link[AirPocket[i].firstpipe].node1].newDepth + Node[Link[AirPocket[i].firstpipe].node1].invertElev,
			nodetoconsidermin = Node[Link[AirPocket[i].firstpipe].node1].newDepth + Node[Link[AirPocket[i].firstpipe].node1].invertElev;

		nodetoconsidermax = MAX(nodetoconsidermax, Node[Link[AirPocket[i].firstpipe].node2].newDepth + Node[Link[AirPocket[i].firstpipe].node2].invertElev);
		nodetoconsidermax = MAX(nodetoconsidermax, Node[Link[AirPocket[i].lastpipe].node1].newDepth + Node[Link[AirPocket[i].lastpipe].node1].invertElev);
		nodetoconsidermax = MAX(nodetoconsidermax, Node[Link[AirPocket[i].lastpipe].node2].newDepth + Node[Link[AirPocket[i].lastpipe].node2].invertElev);

		nodetoconsidermin = MAX(nodetoconsidermin, Node[Link[AirPocket[i].firstpipe].node2].newDepth + Node[Link[AirPocket[i].firstpipe].node2].invertElev);
		nodetoconsidermin = MAX(nodetoconsidermin, Node[Link[AirPocket[i].lastpipe].node1].newDepth + Node[Link[AirPocket[i].lastpipe].node1].invertElev);
		nodetoconsidermin = MAX(nodetoconsidermin, Node[Link[AirPocket[i].lastpipe].node2].newDepth + Node[Link[AirPocket[i].lastpipe].node2].invertElev);

		double elevation = (Node[Link[AirPocket[i].x_mass_pipe].node2].invertElev + Node[Link[AirPocket[i].x_mass_pipe].node1].invertElev) / 2;

		AirPocket[i].currentAirPressure = ((ATM_PRESSURE + ((nodetoconsidermax + nodetoconsidermin) / 2 - elevation) * MperFT * 9800));


		for (int size = 0; size < AirPocket[i].npipes; size++)
		{
			if (AirPocket[i].pipes[size] != -1)
			{
				Link[AirPocket[i].pipes[size]].airpocketvolume_trapped = airVolumeinPipe(AP_inAnalysis[size]);
				Link[AirPocket[i].pipes[size]].correspondingWaterDepth = xsect_getYofA(&Link[AP_inAnalysis[size]].xsect, ((Link[AP_inAnalysis[size]].xsect.aFull * Conduit[AP_inAnalysis[size]].length) - AirPocket[i].currentAirVolume / AirPocket[AirPocketCounter].npipes) / Conduit[AP_inAnalysis[size]].length); //area de seco com ar
				Link[AirPocket[i].pipes[size]].airpressure = AirPocket[AirPocketCounter].currentAirPressure;
				Link[AirPocket[i].pipes[size]].checked = 1;
				Link[AirPocket[i].pipes[size]].newAirPocket = 1;
			}
		}
	}

	return 0;
}

int swmm_runAirPocketInit()
{
	return 0;
}

double airVolumeinPipe(int i)  // returns the air volume inside a certain pipe
{
	double h_up, h_down, air_volume, Area_up, Area_down, D, L, x, Area_full;
	int node_up, node_down;
	air_volume = 0.0;

	if (Link[i].type == CONDUIT)
	{
		node_up = Link[i].node1;
		node_down = Link[i].node2;
		if (Node[node_up].newDepth > Link[i].xsect.yFull && Link[i].correspondingWaterDepth != 0 && Link[i].newAirPocket != 0)
			h_up = Link[i].correspondingWaterDepth;
		else
			h_up = Node[node_up].newDepth;
		if (Node[node_down].newDepth > Link[i].xsect.yFull && Link[i].correspondingWaterDepth != 0 && Link[i].newAirPocket != 0)
			h_down = Link[i].correspondingWaterDepth;
		else
			h_down = Node[node_down].newDepth;
		Area_full = Link[i].xsect.aFull;
		D = Link[i].xsect.yFull;
		L = Conduit[i].length;

		if (h_up < D && h_down < D)
		{
			Area_up = xsect_getAofY(&Link[i].xsect, h_up);
			Area_down = xsect_getAofY(&Link[i].xsect, h_down);
			air_volume += ((Area_full - Area_down) + (Area_full - Area_up)) / 2 * L + Link[i].airpocketvolume_trained;
		}
		else if (h_up >= D && h_down <= D)
		{
			Area_down = xsect_getAofY(&Link[i].xsect, h_down);
			x = (D - h_down) * L / (h_up - h_down);
			air_volume += (Area_full - Area_down) / 3 * x + Link[i].airpocketvolume_trained;
		}
		else if (h_up < D && h_down > D)
		{
			Area_up = xsect_getAofY(&Link[i].xsect, h_up);
			x = (D - h_up) * L / (h_down - h_up);
			air_volume += (Area_full - Area_up) / 3 * x + Link[i].airpocketvolume_trained;
		}
		else if (h_up > D && h_down > D)
		{
			air_volume = 0.0 + Link[i].airpocketvolume_trained;
		}
	}
	else
		("\n Link %d is not a pipe \n ", Link[i].subIndex);
	if (air_volume < 0.0) {
		air_volume = 0.0;
		printf("\n Error in volume in Link: %d \n", Link[i].subIndex);
	}

	return air_volume;
}